/*------------------------------------------------------------------------
 Module:        D:\tbrststs\tbrststs.c
 Author:        A.P. Veening
 Project:       Thunderbird Reset Status
 State:
 Creation Date: 30-08-2006
 Description:   Copy an Mbox file and change the X-Mozilla-Status value
 				to all zeroes.

 Linux version by Paolo "Kaosmos"
 Creation Date: 24-07-2007
------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdbool.h>

FILE *mbox_in;
char line_in[4096];

int get_line_in(void)
{
	int  i = 0;
	int  c;
	int  result = 0;
	char cn;
	char c0;

	memcpy(&cn,"\n",1);
	memcpy(&c0,"\0",1);
	memset(line_in,0,sizeof(line_in));

	c = fgetc(mbox_in);
	if (c == cn) {
		line_in[i] = c;
		i++;
		result = 1;
	}
	while (result == 0) {
		if (c != c0) {
			line_in[i] = c;
			i++;
		}
		if (i < sizeof(line_in)) {
			c = fgetc(mbox_in);
			if (c == EOF) {
				if (i > 0) {
					result = 2;
				} else {
					result = 3;
				}
			} else if (c == cn) {
				line_in[i] = c;
				i++;
				result = 1;
			}

		} else {
			result = 1;
		}
	}
	return result;
}

int main(int argc, char *argv[])
{
	if(argc < 2 || argc > 3 || (argc == 3 && strcmp(argv[1], "-x")) )
	{
		fprintf(stderr, "\nThunderbird-Reset-Status for Linux v.0.2\nUsage:  thunderbird-reset-status [-x] <filename>\nWith -x parameter the output file will have just\nthe deleted messages with the reset status.\nThe output file is named <filename>.mbx\n\n");
		return 1;
	}

	if (strlen(argv[1]) > 254)
	{
		fprintf(stderr,"\nError: filename too long (max 255 chars)\n");
		return 2;
  	}

	char file_name_in[255];
	char file_name_out[255];
	char file_name_simple[255];
	int  rec_status = 5;
	bool file_out_open = false;
	FILE *mbx_out;
	char * pch;
	int offset;
	int mode = 0;

	char buffer[1000];
	// status = 0 --> write in "buffer"
	// status = 1 --> write in file
	// status = 2 --> don't write 
	// used just with mode = 1 (-x mode) 
	int status;
	int rows = 0;

	sprintf(buffer,"");
	if (argc == 3)
	{
		mode = 1;	
		strcpy(file_name_in, argv[2]);
	}
	else
	{
		strcpy(file_name_in, argv[1]);
	}
	
	/* Strip the path and find the simple filename */	
	pch=strrchr(file_name_in,'/');
	if (pch != NULL) {
		offset = pch-file_name_in+1;
	}
	else {
		offset = 0;
	}
	sprintf (file_name_simple,"%s",&file_name_in[offset]);

	mbox_in = fopen(file_name_in,"r");
	if (mbox_in == NULL) {
		mbox_in = fopen(file_name_in,"r");
		if (mbox_in == NULL) {
			printf("\nFile '%s' was not found or locked.\n", file_name_in);
			exit(1);
		}
	}

	rec_status = get_line_in ();
	while (rec_status == 1) {
		if (!file_out_open) {
			memcpy (file_name_out, file_name_simple, sizeof(file_name_simple));
			strcat (file_name_out, ".mbx");
			mbx_out = fopen(file_name_out,"w");
			file_out_open = true;
		}

		if (mode == 1 && memcmp(line_in,"From - ",7) == 0) {
			status = 0;
		}

		// If no X-Mozilla-Status is found in first 6 rows, skip the message
		// It shouldn't happen, but avoid buffer overflow in "buffer"
		if (rows > 6) {
			sprintf(buffer,"");
			status = 2;
			rows = 0;
		}

		if (memcmp(line_in,"X-Mozilla-Status: ",18) == 0) {
			if (mode == 0) {
				memcpy (line_in, "X-Mozilla-Status: 0000\r\n",24);
			}
			else 
			{
				if (memcmp(&line_in[21],"8",1) == 0) {
					memcpy (&line_in[21], "0",1);
					fputs(buffer, mbx_out);					
					status = 1;
				}
				else if (memcmp(&line_in[21],"9",1) == 0) {
					memcpy (&line_in[21], "1",1);
					fputs(buffer, mbx_out);					
					status = 1;
				}
				 else if (memcmp(&line_in[21],"a",1) == 0) {
					memcpy (&line_in[21], "2",1);
					fputs(buffer, mbx_out);					
					status = 1;
				}
				else if (memcmp(&line_in[21],"b",1) == 0) {
					memcpy (&line_in[21], "3",1);
					fputs(buffer, mbx_out);					
					status = 1;
				}
				else if (memcmp(&line_in[21],"c",1) == 0) {
					memcpy (&line_in[21], "4",1);
					fputs(buffer, mbx_out);					
					status = 1;
				}
				else if (memcmp(&line_in[21],"d",1) == 0) {
					memcpy (&line_in[21], "5",1);
					fputs(buffer, mbx_out);					
					status = 1;
				}
				else if (memcmp(&line_in[21],"e",1) == 0) {
					memcpy (&line_in[21], "6",1);
					fputs(buffer, mbx_out);					
					status = 1;
				}
				else if (memcmp(&line_in[21],"f",1) == 0) {
					memcpy (&line_in[21], "7",1);
					fputs(buffer, mbx_out);					
					status = 1;
				}
				else {
					status = 2;
				}
			sprintf(buffer,"");
			rows = 0;
			}
		}

		if (mode == 0 || status == 1)
		{
			fputs(line_in, mbx_out);
		}
		if (mode == 1 && status == 0)
		{
			strcat(buffer,line_in);
			rows++;
		}
		rec_status = get_line_in ();
	}

	if (rec_status == 2) {
		if (strlen(line_in) > 1) {
			line_in[strlen(line_in) - 1] = 0; // removing artefact x'FF'
			fputs(line_in, mbx_out);
		}
	}

	if (file_out_open) {
		fclose(mbx_out);
	}

	fclose(mbox_in);
	return 0;
}
// Done




